function includes(arr, val) {
  return arr.some(function (el) {
    return el == val;
  })
}

function Guard(params) {
  var opts = params || {};
  this.userProperty = opts.userProperty || 'user';
  this.permissionsProperty = opts.permissionsProperty || 'permissions';
  this.ForbiddenError = opts.ForbiddenError || Error;
}

Guard.prototype._getPermissions = function getPermissions(req) {
  var user = req[this.userProperty];
  if (!user) return false;
  var permissions = user[this.permissionsProperty];
  if (!permissions) return false;
  return Array.isArray(permissions) ? permissions : [ permissions ];
}

Guard.prototype.every = function every(vals) {
  var self = this;
  var allowed = Array.isArray(vals) ? vals : [ vals ];
  return function guard(req, res, next) {
    var permissions = self._getPermissions(req);
    if (!permissions) return next(new Error());
    var pass = allowed.every(function(perm) {
      return includes(permissions, perm);
    });
    if (!pass) return next(new Error);
    next();
  }
}

Guard.prototype.some = function some(vals) {
  var self = this;
  var allowed = Array.isArray(vals) ? vals : [ vals ];
  return function guard(req, res, next) {
    var permissions = self._getPermissions(req);
    if (!permissions) return next(new Error());
    var pass = allowed.some(function(perm) {
      return includes(permissions, perm);
    });
    if (!pass) return next(new Error);
    next();
  }
}

module.exports = function guard(opts) {
  return new Guard(opts);
};
